import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:midtrans_helper/midtrans_router.dart';
import 'package:midtrans_helper/qr_code.dart';

import 'package:request_api_helper/request_api_helper.dart';

class MidtransPay {
  String id, url;
  Function getStatus;

  ///```
  ///[{
  ///   "id": "a01",
  ///   "price": 6000,
  ///   "quantity": 1,
  ///   "name": "Apple"
  /// }],
  ///```
  List itemDetail;

  /// ```
  /// {
  /// "first_name": "Budi",
  /// "last_name": "Susanto",
  /// "email": "budisusanto@example.com",
  /// "phone": "+628123456789",
  /// "billing_address": {
  ///   "first_name": "Budi",
  ///   "last_name": "Susanto",
  ///   "email": "budisusanto@example.com",
  ///   "phone": "08123456789",
  ///  "address": "Sudirman No.12",
  ///  "city": "Jakarta",
  ///  "postal_code": "12190",
  ///  "country_code": "IDN"
  /// }
  /// ```
  Map<String, dynamic> customerDetail;

  /// ```
  /// {
  ///   "first_name": "Budi",
  ///   "last_name": "Susanto",
  ///   "email": "budisusanto@example.com",
  ///   "phone": "0812345678910",
  ///   "address": "Sudirman",
  ///   "city": "Jakarta",
  ///   "postal_code": "12190",
  ///   "country_code": "IDN"
  /// }
  /// ```
  Map<String, dynamic> alamatPengiriman;

  /// ```
  /// {
  ///   "order_id": "CustOrder-110",
  ///   "gross_amount": 9000,
  /// },
  /// ```
  Map<String, dynamic> detailTransaksi;
  /// do something is success
  Function onSuccess;
  /// do something is error
  Function onError;

  /// midtrans server key
  String serverKey;

  MidtransPay({
    this.id,
    this.url,
    this.getStatus,
    this.itemDetail,
    this.customerDetail,
    this.alamatPengiriman,
    @required this.detailTransaksi,
    @required this.serverKey,
    this.onSuccess,
    this.onError
  });

  send(context) async {
    String encoded = utf8.fuse(base64).encode('$serverKey :');
    
    dynamic data = {};
    // if(id == 'gopay'){
    //   data.addAll({"payment_type": "gopay"});
    // }else if(id == 'bca'){
    //   data.addAll({"payment_type": "bca"});
    // }
    
    data.addAll({
      "transaction_details": detailTransaksi
    });

    if(itemDetail is Map){
      data.addAll({"item_details": itemDetail});
    }

    if(customerDetail is Map){
      data.addAll({"customer_details": customerDetail});
    }

    if(customerDetail is Map){ 
      if(alamatPengiriman is Map){
        customerDetail.addAll({"shipping_address" : alamatPengiriman});
      }
    }

    switch (id) {
      case 'permata':
        {
          data.addAll({
            "payment_type": "bank_transfer",
            "bank_transfer": {"bank": "permata"}
          });
        }
        break;
      case 'bni':
        {
          data.addAll({
            "payment_type": "bank_transfer",
            "bank_transfer": {"bank": "bni"}
          });
        }
        break;
      case 'bca':
        {
          data.addAll({
            "payment_type": "bank_transfer",
            "bank_transfer": {
              "bank": "bca",
            }
          });
        }
        break;
      case 'bri':
        {
          data.addAll({
            "payment_type": "bank_transfer",
            "bank_transfer": {
              "bank": "bri",
            }
          });
        }
        break;
      case 'mandiri':
        {
          data.addAll({"payment_type": "echannel"});
        }
        break;
      case 'visa':
        {
          data.addAll({"payment_type": "null"});
        }
        break;
      case 'american':
        {
          data.addAll({"payment_type": "null"});
        }
        break;
        break;
      case 'master':
        {
          data.addAll({"payment_type": "null"});
        }
        break;
        break;
      case 'jcb':
        {
          data.addAll({"payment_type": "null"});
        }
        break;
      case 'akulaku':
        {
          data.addAll({"payment_type": "akulaku"});
        }
        break;
      case 'gopay':
        {
          data.addAll({"payment_type": "gopay"});
        }
        break;
      case 'indomart':
        {
          data.addAll({
            "payment_type": "cstore",
            "cstore": {"store": "Indomaret", "message": "Message to display"}
          });
        }
        break;
      case 'alfamart':
        {
          data.addAll({
            "payment_type": "cstore",
            "cstore": {
              "store": "alfamart",
              "alfamart_free_text_1": "1st row of receipt,",
              "alfamart_free_text_2": "This is the 2nd row,",
              "alfamart_free_text_3": "3rd row. The end."
            },
          });
        }
        break;
      case 'bcalklikpay':
        {
          data.addAll({
            "payment_type": "bca_klikpay",
            "bca_klikpay": {"type": 1, "description": "Pembelian Barang"}
          });
        }
        break;
      case 'cimbclicks':
        {
          data.addAll({
            "payment_type": "cimb_clicks",
            "cimb_clicks": {"description": "Purchase of a Food Delivery"}
          });
        }
        break;
      case 'danamonbanking':
        {
          data.addAll({"payment_type": "danamon_online"});
        }
        break;
      case 'epaybri':
        {
          data.addAll({"payment_type": "bri_epay"});
        }
        break;
    }
    

    dynamic body = json.encode(data);
    Map<String, String> header = {
      'Accept': 'application/json',
      'Authorization': 'Basic $encoded',
      'Content-Type': 'application/json',
    };

    await Post(
      customHeader: header,
      customUrl: '${url}charge',
      body: body,
      withLoading: true,
      socketMessage: 'error connection',
      onError: (code,data){
        print(code);
        print(data);
      },
      onSuccess: (data) async {
        if(onSuccess != null){
          return onSuccess(data);
        }
        if(data['status_code'].toString() == '201'){
          print(data);
          await router(context, id, data);
        }else{
          if(onError != null){
            return onError(data);
          }
          print(data);
        }
      },
      exception: true,
      onException: (error) async {
        await Post(
          customHeader: header,
          customUrl: '${url}charge',
          body: utf8.encode(body),
          withLoading: true,
          socketMessage: 'error connection',
          onError: (code,data){
            print(code);
            print(data);
          },
          onSuccess: (data) async {
            if(onSuccess != null){
              return onSuccess(data);
            }
            if(data['status_code'].toString() == '201'){
              await router(context, id, data);
            }else{
              if(onError != null){
                return onError(data);
              }
              print(data);
            }
          },
          exception: true,
          onException: (error){
            print(error.toString());
          },
          timeoutMessage: 'timeout',
        ).request(context);
      },
      timeoutMessage: 'timeout',
    ).request(context);
  }
}
